@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from pydantic import BaseModel
 
 
@@ -6,4 +8,6 @@ class LinkPreview(BaseModel):
     title: str
     description: str
     url: str
-    id: str | None = None  #  This is the local filename for a received link preview
+
+    #  This is the local filename for a received link preview
+    id: str | None = None
